(function() {
  var language = {};
  var preference = {};
  var varCache = {
    icons: {},
    activeTabInfo: {}
  };
  var menuContainer = undefined;

  var translatePage = function() {
    var elList = menuContainer.querySelectorAll('*[data-i18n]');
    for(var i = 0, len = elList.length; i < len; i++) {
      var el = elList[i];
      var key = el.dataset.i18n;
      el.textContent = language[key];
      if (el.classList.contains('label')) {
        el.title = language[key];
      }
    }
  };

  var onModuleToggle = function() {
    var state = varCache.moduleTrigger.classList.contains('disabled') ? 1 : 0;
    setCheckboxState(state);
    var tabInfo = varCache.activeTabInfo;
    tabInfo.state = state;
    if (Array.isArray(tabInfo.prefKey)) {
      for (var i = 0, key; key = tabInfo.prefKey[i]; i++) {
        preference[key] = state;
      }
    } else {
      preference[tabInfo.prefKey] = state;
    }
    mono.sendMessage({action: 'viaMenu_' + 'changeState', state: state ?  1 : 0, prefKey: tabInfo.prefKey, moduleName: tabInfo.moduleName, needInclude: tabInfo.isNotResponse});
    tabInfo.isNotResponse = false;
    updateMenuItem(tabInfo);
  };

  var menuItemAction = function(event) {
    event.preventDefault();
    event.stopPropagation();
    var node = this;

    if (node.classList.contains('inactive')) {
      return;
    }

    var action = node.dataset.action;

    var isModule = node.classList.contains('module');

    if ([1].indexOf(preference.cohortIndex) !== -1) {
      if (['updateLinks', 'downloadPlaylist', 'downloadPhotos', 'downloadMP3Files'].indexOf(action) !== -1) {
        if (['dailymotion', 'facebook', 'mailru', 'odnoklassniki', 'savefrom', 'soundcloud', 'vimeo', 'vk',
            'youtube',
            'instagram', 'rutube'].indexOf(varCache.activeTabInfo.moduleName) !== -1) {
          mono.sendMessage({action: 'trackCohort', category: 'extensionMenu', event: 'click', label: action});
        } else {
          mono.sendMessage({action: 'trackCohort', category: 'sitesNotSupporded', event: action, label: '%domain%'});
        }
      }
    }

    mono.sendMessage({action: 'trackEvent', category: 'extensionMenu', event: 'click', label: action});

    if (action === 'enableModule') {
      return onModuleToggle();
    } else {
      mono.sendMessage({action: (isModule ? 'viaMenu_' : '') + action});
    }

    if (mono.isGM) {
      _menu.hideMenuItems();
      return;
    }
    if (mono.isFF) {
      mono.addon.postMessage('hidePopup');
      return;
    }
    if (mono.isSafari) {
      safari.extension.popovers[0].hide();
      return;
    }
    window.close();
  };

  var updateMenu = function() {
    if (!menuContainer) {
      return;
    }

    updateDescription('showAboutPage');

    mono.sendMessage('getPreference', function(resp) {
      preference = resp;
      updateMenuItem();
    });

    mono.msgClean();
  };

  var setItemState = function (className, show) {
    var li = menuContainer.querySelectorAll('div.' + className);
    for(var i = 0; i < li.length; i++) {
      if (show) {
        li[i].classList.remove('inactive');
      } else {
        li[i].classList.add('inactive');
      }
    }
  };

  var safariBlankLink = function() {
    mono.openTab(this.href, true);
  };

  var updateSafariLinks = function() {
    var links = menuContainer.querySelectorAll('a[href][target="_blank"]');
    for (var i = 0, len = links.length; i < len; i++) {
      links[i].removeEventListener('click', safariBlankLink);
      links[i].addEventListener('click', safariBlankLink);
    }
  };

  var initSocial = function() {
    var targetUrl = 'http://savefrom.net/user.php';
    var url = encodeURIComponent(targetUrl);
    var image = encodeURIComponent('http://savefrom.net/img/icon_100.png');
    var title = encodeURIComponent(language.extName);
    var desc = encodeURIComponent(language.socialDesc);
    var btnList = {
      vk: {
        network: 'vkontakte',
        title: language.shareIn.replace('%w', 'VK'),
        href: 'http://vk.com/share.php?url='+url+'&image='+image+'&title='+title+'&description='+desc
      },
      ok: {
        network: 'odnoklassniki',
        title: language.shareIn.replace('%w', 'OK.ru'),
        href: 'http://www.odnoklassniki.ru/dk?st.cmd=addShare&st.s=1&st._surl='+url+'&st.comments='+desc
      },
      mailru: {
        network: 'mail.ru',
        title: language.shareIn.replace('%w', 'Mail.ru'),
        href: 'http://connect.mail.ru/share?url='+url+'&title='+title+'&description='+desc+'&imageurl='+image
      },
      tw: {
        network: 'twitter',
        title: language.shareIn.replace('%w', 'Twitter'),
        href: 'https://twitter.com/intent/tweet?text='+title+'&url='+url
      },
      fb: {
        network: 'facebook',
        title: language.shareIn.replace('%w', 'Facebook'),
        href: 'http://www.facebook.com/sharer.php?s=100&p[url]='+url+'&p[title]='+title+'&p[summary]='+desc+'&p[images][0]='+image
      },
      gp: {
        network: 'google+',
        title: language.shareIn.replace('%w', 'Google+'),
        href: 'https://plus.google.com/share?url='+url
      },
      lj: {
        network: 'livejournal',
        title: language.shareIn.replace('%w', 'Livejournal'),
        href: 'http://www.livejournal.com/update.bml?subject='+title+'&event='+desc+' '+url
      }
    };
    for (var item in btnList) {
      var link = menuContainer.querySelector('.social-btn.'+item);
      if (!link) {
        continue;
      }
      link.title = btnList[item].title;
      link.href = btnList[item].href;
      link.dataset.network = btnList[item].network;
    }
    link.parentNode.addEventListener('click', function(e) {
      var btn = e.target;
      if (!btn.classList.contains('social-btn')) {
        return;
      }
      var network = btn.dataset.network;
      mono.sendMessage({action: 'trackSocial', target: targetUrl, event: 'share', network: network});
    });
  };

  var setModuleBtnState = function(tabInfo) {
    setItemState(tabInfo.moduleName, true);

    if (tabInfo.moduleName === 'youtube') {
      setItemState('plYoutube', false);
      var isPlaylist = false;
      var url = tabInfo.url;
      if (url.indexOf('/playlist?') !== -1) {
        isPlaylist = true;
      } else
      if ((/[?&]{1}list=/).test(url)) {
        isPlaylist = true;
      } else {
        var matched = url.match(/(user|channel|show)\/[^\/]+(\/feed|\/featured|\/videos|$)/i);
        if (!matched) {
          matched = url.match(/\/(feed)\/(trending|subscriptions|history)/i);
        }
        if (!matched || matched.length < 3) {
          isPlaylist = false;
        } else {
          isPlaylist = true;
        }
      }
      if (isPlaylist) {
        setItemState('plYoutube', true);
      }
    }
  };

  var onGetTabInfo = function(tabInfo, force) {
    varCache.activeTabInfo = tabInfo;

    if (tabInfo.moduleName) {
      setCheckboxState(tabInfo.state);
      setItemState('enableModule', true);
    }

    if (['odnoklassniki'].indexOf(tabInfo.moduleName) !== -1) {
      setItemState('bookmarklet', false);
    }

    if (tabInfo.state) {
      setModuleBtnState(tabInfo);
    }

    if (tabInfo.moduleName === 'savefrom') {
      setCheckboxState('force');
      setItemState('enableModule', false);
    }

    if (!force) {
      tabInfo.isNotResponse = true;
      mono.sendMessage({action: 'getActiveTabModuleInfo', url: tabInfo.url}, function(moduleInfo) {
        tabInfo.isNotResponse = false;
        if (moduleInfo.moduleName !== tabInfo.moduleName) return;
        moduleInfo.url = tabInfo.url;

        setCheckboxState(moduleInfo.state);

        if (moduleInfo.state) {
          setModuleBtnState(moduleInfo);
        } else {
          setItemState('module', false);
        }
      });
    }
  };

  var updateMenuItem = function(moduleInfo) {
    setItemState('module', false);
    setItemState('bookmarklet', true);
    setItemState('enableModule', false);
    setCheckboxState(0);

    varCache.activeTabInfo = {};

    if (moduleInfo) {
      onGetTabInfo(moduleInfo, 1);
    } else {
      mono.sendMessage('getActiveTabInfo', onGetTabInfo);
    }
  };

  var setIconState = function(el, state) {
    var type = el.getAttribute('data-type');
    var path = el.querySelector('path');
    if (state === 'hover') {
      path.setAttribute('fill', '#ffffff');
    } else
    if (state === 'active') {
      path.setAttribute('fill', '#AAAAAA');
    } else
    if (state) {
      if (type === 'downloadMP3Files') {
        path.setAttribute('fill', '#00CCFF');
      } else
      if (type === 'downloadPlaylist') {
        path.setAttribute('fill', '#77D1FA');
      } else
      if (type === 'downloadPhotos') {
        path.setAttribute('fill', '#88cb66');
      } else
      if (type === 'showAboutPage') {
        path.setAttribute('fill', '#ADE61B');
      } else
      if (type === 'updateLinks') {
        path.setAttribute('fill', '#CB7FBD');
      } else
      if (type === 'downloadFromCurrentPage') {
        path.setAttribute('fill', '#CB7FBD');
      }
    }
  };

  var updateDescription = function(action, label) {
    var desc = varCache.desc;
    var text = varCache.deskText;
    var title = varCache.descTitel;
    var more = varCache.descMore;
    desc.dataset.page = action;

    var icon = varCache.icons[ action ];
    var subIcon = desc.querySelector('.icon');
    if (icon) {
      var _icon = icon.cloneNode(true);
      setIconState(_icon, 'active');
      if (subIcon) {
        subIcon.parentNode.replaceChild(_icon, subIcon);
      }
      subIcon.style.visibility = 'visible';
    } else {
      subIcon.style.visibility = 'hidden';
    }
    if (action === 'showAboutPage') {
      title.textContent = language.aboutTitle;
      text.textContent = '';
      var aboutText = mono.create('p', {text: language.aboutDescription});
      var linkSupport = mono.create('a', {href: 'http://savefrom.net/faq.php#supported_resourses', target: '_blank', text: language.aboutSupported});
      var linkHome = mono.create('a', {href: 'http://savefrom.net/user.php?helper=' + varCache.browser, target: '_blank', text: language.homePage});
      linkSupport.style.display = 'block';
      linkHome.style.display = 'block';
      text.appendChild(aboutText);
      text.appendChild(linkSupport);
      text.appendChild(linkHome);
      more.style.display = 'none';
    } else {
      title.textContent = label;
      text.textContent = language['menu'+mono.capitalize(action)] || '';
      more.style.display = 'block';
    }
    if (mono.isSafari) {
      updateSafariLinks();
    }
  };

  var onReady = function() {
    translatePage();

    varCache.descMore.href = 'http://savefrom.net/user.php?helper=' + varCache.browser;

    var el = menuContainer.querySelectorAll('div[data-action]');
    for(var i = 0; i < el.length; i++) {
      var icon = el[i].querySelector('svg');
      if (icon) {
        varCache.icons[ el[i].dataset.action ] = icon;
        setIconState(icon, 1);
      }
      if (el[i].style.display === 'none') continue;
      el[i].addEventListener('click', menuItemAction, false);
      mono.on(el[i], 'mouseenter', function(e) {
        var action = this.dataset.action;
        var icon = varCache.icons[ action ];
        if (icon) {
          setIconState(icon, 'hover');
        }
        var span = this.querySelector('span');
        var title = '';
        if (span) {
          title = span.textContent;
        }
        updateDescription(action, title);
      });
      mono.on(el[i], 'mouseleave', function(e) {
        var action = this.dataset.action;
        var icon = varCache.icons[ action ];
        if (icon) {
          setIconState(icon, 1);
        }
      });
    }

    initSocial();
    if (mono.isSafari) {
      updateSafariLinks();
    }

    updateDescription('showAboutPage');

    updateMenuItem();

    menuContainer.classList.remove('loading');
  };

  var setCheckboxState = function(state) {
    if (state === 'force') {
      varCache.moduleTrigger.classList.add('enableForce');
    } else {
      varCache.moduleTrigger.classList.remove('enableForce');
    }
    if (state) {
      varCache.moduleTrigger.classList.remove('disabled');
      varCache.moduleTrigger.nextElementSibling.textContent = language.disableModule;
    } else {
      varCache.moduleTrigger.classList.add('disabled');
      varCache.moduleTrigger.nextElementSibling.textContent = language.enableModule;
    }
    if (varCache.desc.dataset.page !== 'showAboutPage') {
      updateDescription('enableModule', state ? language.disableModule : language.enableModule);
    }
  };

  var tutorial = {
    show: function() {
      "use strict";
      if (!SaveFrom_Utils.tutorialTooltip) {
        return;
      }

      if (!preference.showTutorial) {
        return;
      }

      if (mono.isGM) {
        mono.storage.set({onceYtTutorial: 1});
        return;
      }

      mono.storage.get('onceYtTutorial', function(storage) {
        if (storage.onceYtTutorial) {
          return;
        }
        storage.onceYtTutorial = 1;

        SaveFrom_Utils.tutorial.show({
          container: menuContainer,
          width: 482,
          height: 404 + (mono.isGM ? 2 : 0),
          padding: 4,
          slideList: SaveFrom_Utils.tutorial.getYtSlideList('black'),
          onClose: function() {
            mono.storage.set(storage);

            mono.sendMessage({action: 'setIconBadge', text: ''});
          },
          checkExists: function(cb) {
            mono.storage.get('onceYtTutorial', function(storage) {
              if (storage.onceYtTutorial) {
                return cb(1);
              }
              cb();
            });
          },
          trackId: 'Menu',
          boxStyle: {
            backgroundColor: 'transparent'
          },
          containerStyle: {
            borderRadius: '3px',
            backgroundColor: 'rgba(0, 104, 255, 0.9)',
            padding: 0,
            margin: '4px',
            boxShadow: 'none'
          },
          slideStyle: {
            backgroundColor: 'transparent',
            borderRadius: 0
          },
          leftBtnStyle: {
            top: '4px',
            left: '4px'
          },
          rightBtnStyle: {
            top: '4px',
            right: '4px'
          },
          closeBtnStyle: {
            backgroundColor: '#fff',
            color: 'rgba(0, 104, 255, 0.9)'
          },
          cssStyle: {
            ' .sf-dots': {
              'paddingTop': '2px'
            },
            ' .sf-dot i': {
              backgroundColor: '#fff'
            },
            ' .sf-dot.active i': {
              backgroundColor: 'transparent',
              borderRadius: '6px',
              margin: '-1px',
              width: '6px',
              height: '6px',
              border: '2px solid #fff'
            },
            ' .sf-slider-conteiner span': {
              color: '#fff !important'
            },
            ' .sf-slider-conteiner a': {
              color: '#fff !important'
            }
          },
          arrowColor: '#fff',
          arrowColorActive: '#fff',
          onResize: function(details) {
            details.box.style.position = 'absolute';
          },
          withOpacity: true,
          withDelay: 250,
          onShow: function() {
            mono.isSafari && updateSafariLinks();

            mono.sendMessage({action: 'setIconBadge', text: '?'});
          }
        });
      });
    }
  };

  var run = function(parent) {
    menuContainer = (parent || document).getElementsByClassName('sf-menu-container')[0];
    if (!menuContainer) {
      return;
    }

    setTimeout(function() {
      menuContainer.classList.remove('loading');
    }, 1000);

    varCache.list = menuContainer.querySelector('.sf-menu-list');
    varCache.desc = menuContainer.querySelector('.sf-menu-desc');
    varCache.descTitel = varCache.desc.querySelector('.title');
    varCache.deskText = varCache.desc.querySelector('.desc');
    varCache.descMore = varCache.desc.querySelector('.more');
    var version = varCache.desc.querySelector('.version');


    varCache.list.style.height = varCache.list.offsetHeight + 'px';

    mono.sendMessage(['getLanguage', 'getPreference', 'getVersion', 'getLastVersionInfo', 'getBrowser'], function(response) {
      language = response.getLanguage;
      preference = response.getPreference;

      mono.global.language = language;
      mono.global.preference = preference;

      varCache.browser = response.getBrowser;

      version.textContent = '';
      mono.create(version, {
        append: [
          mono.create('span', {
            text: language.aboutVersion+' '+response.getVersion
          }),
          !response.getLastVersionInfo ? undefined : mono.create('a', {
            text: language.updateTo.replace('%d', response.getLastVersionInfo),
            href: 'http://savefrom.net/user.php?helper=' + varCache.browser + '&update=' + response.getVersion,
            target: '_blank',
            on: ['click', function() {
              mono.sendMessage({action: 'trackEvent', category: 'extensionMenu', event: 'click', label: 'updateVersion'});
            }]
          })
        ]
      });

      if (['en', 'uk', 'ru'].indexOf(language.lang) === -1) {
        menuContainer.classList.add('no-poll');
      }

      varCache.moduleTrigger = menuContainer.querySelector('.sf-checkbox');

      tutorial.show();

      onReady();
    });
  };

  mono.onReady(function() {
    mono.onMessage(function (mesasge, response) {
      if (mesasge === 'updateMenu') {
        updateMenu();
      }
    });

    if (mono.isGM) {
      _menu.initMenu = run;
    } else
    if (mono.isSafari) {
      safari.application.addEventListener('popover', function () {
        if (!varCache.first) {
          varCache.first = 1;
          run();
        } else {
          updateMenu();
        }
      }, true);
    } else {
      run();
    }
  });

  //@insert
})();