// ==UserScript==
// @exclude *
// ==/UserScript==

var vimeo_com_embed = {
  getVimeoLinks: function(request, callback) {
    "use strict";
    function callback_links(links, title, thumb) {
      var response = {
        action: request.action,
        extVideoId: request.extVideoId,
        links: links,
        title: title,
        thumb: thumb
      };

      callback(response);
    }

    vimeo_com_embed._getVimeoLinks(request.extVideoId, callback_links);
  },
  _getVimeoLinks: function(videoId, callback) {
    "use strict";
    vimeo_com_embed.getVimeoNoEmbedLinks(videoId, function(links, title, thumb){
      if(links) {
        return callback(links, title, thumb);
      }

      vimeo_com_embed.getVimeoEmbedLinks(videoId, callback);
    });
  },
  getVimeoEmbedLinks: function(videoId, callback) {
    "use strict";
    var abort = function() {
      return callback(null, '', '');
    };

    mono.ajax({
      url: 'https://player.vimeo.com/video/' + videoId,
      success: function(data) {
        if (!data) {
          return abort();
        }

        var jsonList = mono.findJson(data, [/"files":/]);
        var config = null;
        jsonList.some(function(obj) {
          "use strict";
          if (!obj.video || !obj.request || !obj.request.files) {
            return;
          }

          config = obj;
          return true;
        });

        data = vimeo_com_embed.getLinksFromConfig(config);
        if (data) {
          return callback(data.links, data.title, data.thumb);
        }

        return abort();
      },
      error: function() {
        return abort();
      }
    });
  },
  getConfig: function(url, cb) {
    "use strict";
    var abort = function() {
      return cb(null, '', '');
    };

    mono.ajax({
      url: url,
      success: function(data) {
        if (!data) {
          return abort();
        }

        data = vimeo_com_embed.getVimeoDataFromConfig(data);
        if(data) {
          return cb(data.links, data.title, data.thumb);
        }

        return abort();
      },
      error: function() {
        return abort();
      }
    });
  },
  getClipPageConfig: function(data, cb) {
    "use strict";
    var abort = function() {
      return cb(null, '', '');
    };

    var configUrl = null;
    var scriptList = mono.getPageScript(data, /['"]config_url['"]\s*:\s*/);
    scriptList.some(function(script) {
      var configList = mono.findJson(script, /['"]config_url['"]\s*:\s*/);
      return configList.some(function(config) {
        if (config.player) {
          configUrl = config.player.config_url;
          if (configUrl) {
            return true;
          }
        }
      });
    });

    if (configUrl) {
      return vimeo_com_embed.getConfig(configUrl, cb);
    }

    return abort();
  },
  getVimeoNoEmbedLinks: function(videoId, cb) {
    "use strict";
    var abort = function() {
      return cb(null, '', '');
    };

    mono.ajax({
      url: 'https://vimeo.com/' + videoId,
      success: function(data) {
        if (!data) {
          return abort();
        }

        var configUrl = data.match(/data-config-url=["']([^\s"'<>]+)/i);
        configUrl = configUrl && configUrl[1].replace(/&amp;/ig, '&');

        if(configUrl) {
          vimeo_com_embed.getConfig(configUrl, cb);
        } else {
          vimeo_com_embed.getClipPageConfig(data, cb);
        }
      },
      error: function() {
        return abort();
      }
    });
  },
  getVimeoLinksFromConfig: function(msg, response) {
    "use strict";
    var data = vimeo_com_embed.getLinksFromConfig(msg.config);
    if(data) {
      return response(data);
    }

    return response(null);
  },
  getLinksFromConfig: function(config) {
    "use strict";
    if (!config || !config.video || !config.request || !config.request.files) {
      return null;
    }

    var video = config.video;
    var files = config.request.files;

    var data = {};

    data.title = video.title || '';

    var maxSize = null;
    for (var size in video.thumbs) {
      if (maxSize === null || maxSize < size) {
        maxSize = size;
        data.thumb = video.thumbs[size];
      }
    }

    data.links = [];

    for (var type in files) {
      if (!Array.isArray(files[type])) {
        continue;
      }
      files[type].forEach(function(item) {
        if (!item || !item.url || !item.mime) {
          return;
        }

        var ext = item.mime.split('/')[1];
        if (!ext) {
          ext = item.url.match(/\.(\w{2,4})(?:\?|#|$)/i);
          ext = ext && ext[1] || 'mp4';
        }

        var name = item.quality;
        if (/^\d+p$/.test(name)) {
          name = name.replace(/p$/, '');
        }

        data.links.push({
          url: item.url,
          name: name,
          type: ext.toUpperCase(),
          format: ext.toUpperCase(),
          ext: ext
        });
      });
    }

    if (!data.links.length) {
      data = null;
    }

    return data;
  },
  getVimeoDataFromConfig: function(config) {
    "use strict";
    config = config.replace(/(\{|,)\s*(\w+)\s*:/ig, '$1"$2":').
      replace(/(:\s+)\'/g, '$1"').replace(/\'([,\]\}])/g, '"$1');

    try {
      config = JSON.parse(config);
    } catch(err) {
      return null;
    }

    return this.getLinksFromConfig(config);
  }
};

if (typeof window === 'undefined') {
  exports.init = function(_mono, _engine) {
    mono = _mono;
    engine = _engine;
    return vimeo_com_embed;
  };
} else {
  engine.modules.vimeo = vimeo_com_embed;
}